with Ada.Real_Time;

package body Activities is

   --  Action is an internal procedure
   --+ that is used to fake workload:
   --+ before using it we need to first declare it
   procedure Action (Load : in     Positive);

   --  and then to specify it
   procedure Action (Load : in     Positive) is
      Factor : constant Positive := 10;
      Dummy  :          Positive := 1;
   begin
      --  this iteration simulates a variable-duration workload
      --+ that may be removed by optimising compilers
      for I in 1 .. Load loop
         Dummy := Dummy + Factor;
         Dummy := Dummy - Factor;
      end loop;
   end Action;

   --  that's the philosophers' reflection procedure
   procedure Think (Think_Load : in     Positive;
                    Some_Time  : in     Positive) is
      use Ada.Real_Time;
   begin
      --  in which first inspiration is sought
      Action (Think_Load);
      --  and then reflection takes place
      delay until Clock + Milliseconds (Some_Time);
   end Think;

   --  that's the philosophers' feeding activity
   procedure Eat (Eat_Load  : in     Positive;
                  Some_Time : in     Positive) is
      use Ada.Real_Time;
   begin
      --  in which first they have a good meal
      Action (Eat_Load);
      --  and then take a well-deserved nap
      delay until Clock + Milliseconds (Some_Time);
   end Eat;

end Activities;
