with Activities;
with Ada.Text_IO;
with Ada.Real_Time;

package body Philosophers is

   --  a classic server task with an open terminate alternative
   --  (note however that the current formulation of the philospher
   --  task makes it non-terminating)
   task body Forks_Control_T is
   begin
      loop
         --  the choice among open alternatives is non-deterministic
         select
            accept Take_Fork;
         or
            terminate;
         end select;
         --+-------------
         select
            accept Put_Fork;
         or
            --  consider alternatives as long as clients exist
            --  otherwise terminate server
            terminate;
         end select;
      end loop;
   end Forks_Control_T;

   --  this server task will accept calls to Enter_Region
   --  only as long as there can arise no deadlock condition
   --  in the distribution of forks to diners
   task body Deadlock_Preventer is
      Eaters : Eaters_T := 0;
   begin
      loop
         select
            when Eaters < Max_Eaters =>
            accept Enter;
            --  we update the count of current eaters
            --  and we do it outside the accept body
            --  because the variable is private
            Eaters := Eaters + 1;
         or
            accept Leave;
            --  as above
            Eaters := Eaters - 1;
         or
            terminate;
         end select;
      end loop;
   end Deadlock_Preventer;

   task body Philosopher_T is
      use Activities,
          Ada.Text_IO;
      Fork_1,
      Fork_2 : Position_T;
      --  count the number of time a Philosopher
      --  has dined (up to constraint error)
      Counter : Natural := 0;
   begin
      Fork_1 := My_Position;
      Fork_2 := Fork_1 + 1;
      loop
         --  first think
         Think (Think_Load, Some_Time);
         --  then feel hungry and wait to seize forks
         --  until there can arise no deadlock
         Deadlock_Preventer.Enter;
         Forks (Fork_1).Take_Fork;
         Forks (Fork_2).Take_Fork;
         --  then eat
         Eat (Think_Load, Some_Time); Counter := Counter + 1;
         --  then relinquish forks
         Forks (Fork_1).Put_Fork;
         Forks (Fork_2).Put_Fork;
         --  decreasing the count of current eaters
         Deadlock_Preventer.Leave;
         --  finally inform us that you are done for one round
         Put_Line ("Philosopher "
                   & Natural'Image (Natural (My_Position))
                   & " has dined "
                   & Counter'Img
                   & " times. Burp!");
      end loop;
   end Philosopher_T;

end Philosophers;
